const path = require('path');
const fs = require('fs');
const jest = require('jest');
const chalk = require('chalk');
const { getJestConfig } = require('build-scripts-config');

const log = require('../utils/log');
const Context = require('../core/Context');

module.exports = async function({
  args,
  rootDir,
}) {
  const command = 'test';

  const context = new Context({
    args,
    command,
    rootDir,
  });
  const { jestArgv = {} } = args || {};
  const { config, regexForTestFiles, ...restArgv } = jestArgv;

  const { applyHook, rootDir: ctxRoot } = context;

  await applyHook(`before.${command}.load`, { args });

  let configArr = [];
  try {
    configArr = await context.setUp();
  } catch (err) {
    log.error(chalk.red('Failed to get config.'));
    await applyHook(`error`, { err });
    throw err;
  }

  if (!configArr.length) {
    const err = new Error('No webpack config found.');
    await applyHook(`error`, { err });
    throw err;
  }
  // get user jest config
  const jestConfigPath = path.join(ctxRoot, config || 'jest.config.js');
    
  let userJestConfig = {};
  if (fs.existsSync(jestConfigPath)) {
    userJestConfig = require(jestConfigPath); // eslint-disable-line
  }

  // get webpack.resolve.alias
  const alias = configArr.reduce((acc, cfg) => {
    if (cfg.resolve) {
      return {
        ...acc,
        ...cfg.resolve.get('alias'),
      };
    } else {
      return acc;
    }
  }, {});

  const aliasModuleNameMapper = {};
  Object.keys(alias || {}).forEach((key) => {
    const aliasPath = alias[key];
    // check path if it is a directory
    if (fs.existsSync(aliasPath) && fs.statSync(aliasPath).isDirectory()) {
      aliasModuleNameMapper[`^${key}/(.*)$`] = `${aliasPath}/$1`;
    }
    aliasModuleNameMapper[`^${key}$`] = aliasPath;
  });

  // generate default jest config
  const jestConfig = getJestConfig({
    rootDir: ctxRoot,
    regexForTestFiles,
    moduleNameMapper: aliasModuleNameMapper,
    userJestConfig,
  });

  // disallow users to modify jest config
  Object.freeze(jestConfig);
  await applyHook(`before.${command}.run`, { args, config: jestConfig });

  const result = await new Promise((resolve, reject) => {
    jest.runCLI(
      {
        ...restArgv,
        config: JSON.stringify(jestConfig),
      },
      [ctxRoot],
    ).then((data) => {
      const { results } = data;
      if (results.success) {
        resolve(data);
      } else {
        reject(new Error('Jest failed'));
      }
    }).catch((err) => {
      log.error(err.message || err);
    });
  });

  await applyHook(`after.${command}`, { result });

  return result;
};
